import React, { useEffect, useRef, useState } from "react";
import FontStrokeOptions from "./TextToolStyles/FontStrokeOptions";
import FontFamilyAndText from "./TextToolStyles/FontFamilyAndText";
import TextSpacings from "./TextToolStyles/TextSpacings";
import MultipleDrawingTools from "./DrawingTools/MultipleDrawingTools";
import AutoSaveDrawing from "./AutoSaveDrawings";
import CollectEmails from "../Emails/CollectEamils";

const MainCanvas = () => {
  const canvasRef = useRef(null);
  const [canvas, setCanvas] = useState(null);
  const [startDrawing, setstartDrawing] = useState(false);
  const [selectedTool, setSelectedTool] = useState();
  const [DrawColor, setDrawColor] = useState("black");
  const [showModal, setShowModal] = useState(false);
  const [drawColorSelectType, setdrawColorSelectType] = useState("stroke");
  const [canvasSize, setCanvasSize] = useState({
    width: typeof window !== "undefined" && window.innerWidth,
    height: typeof window !== "undefined" && window.innerHeight,
  });
  const toggleModal = () => setShowModal(!showModal);
  // States For modals open & close
  const [openSideBarSpacing, setOpenSideBarSpacing] = React.useState(false);
  const [openSideBarFontFamily, setopenSideBarFontFamily] =
    React.useState(false);
  const [openSideBarStroke, setopenSideBarStroke] = React.useState(false);
  //  setWiperWidth
  const [WiperWidth, setWiperWidth] = useState(10);
  // Canvas sizes
  const sizes = {
    socialMedia: {
      instagramPost: { width: 1080, height: 1080, name: "Instagram Post" },
      instagramStory: { width: 1080, height: 1920, name: "Instagram Story" },
      facebookPost: { width: 1200, height: 630, name: "Facebook Post" },
      twitterPost: { width: 1200, height: 675, name: "Twitter Post" },
      linkedinPost: { width: 1200, height: 627, name: "LinkedIn Post" },
      youtubeThumb: { width: 1280, height: 720, name: "YouTube Thumbnail" },
    },
    digitalDevices: {
      smartphoneSmall: { width: 360, height: 640, name: "Smartphone (Small)" },
      smartphoneMedium: {
        width: 414,
        height: 896,
        name: "Smartphone (Medium)",
      },
      smartphoneLarge: { width: 428, height: 926, name: "Smartphone (Large)" },
      tabletPortrait: { width: 768, height: 1024, name: "Tablet (Portrait)" },
      tabletLandscape: { width: 1024, height: 768, name: "Tablet (Landscape)" },
      laptop: { width: 1366, height: 768, name: "Laptop" },
      desktop: { width: 1920, height: 1080, name: "Desktop (1080p)" },
      _4k: { width: 3840, height: 2160, name: "4K Display" },
    },
    paper: {
      a5: { width: 1748, height: 2480, name: "A5 (148 x 210 mm)" },
      a4: { width: 2480, height: 3508, name: "A4 (210 x 297 mm)" },
      a3: { width: 3508, height: 4961, name: "A3 (297 x 420 mm)" },
      usLetter: { width: 2550, height: 3300, name: "US Letter (8.5 x 11 in)" },
      usLegal: { width: 2550, height: 4200, name: "US Legal (8.5 x 14 in)" },
      tabloid: { width: 3300, height: 5100, name: "Tabloid (11 x 17 in)" },
    },
  };

  const changeStrokeSize = (e) => {
    let size = e.target.value;
    let activeObject = canvas.getActiveObject();
    if (activeObject && activeObject?.type !== "i-text") {
      activeObject.set("strokeWidth", size);
    }
    canvas.renderAll();
  };

  const changeWiperWidth = (e) => {
    let size = Number(e.target.value);
    setWiperWidth(size);
    canvas.renderAll();
    handleSelectTool("Wipe");
  };

  const handleSizeChange = (size) => {
    setSelectedTool(!selectedTool);
    setCanvasSize({ width: size.width, height: size.height });
    canvas.setWidth(size.width);
    canvas.setHeight(size.height);
  };

  const handleCustomSizeChange = (e) => {
    const { name, value } = e.target;
    setCustomSize((prev) => ({ ...prev, [name]: value }));
  };

  const applyCustomSize = (e) => {
    e.preventDefault();
    const width = parseInt(customSize.width, 10);
    const height = parseInt(customSize.height, 10);
    if (width > 0 && height > 0) {
      setCanvasSize({ width, height });
      canvas.setWidth(width);
      canvas.setHeight(height);
    }
  };

  const renderSizeButtons = (category) => (
    <div className="space-y-2">
      {Object.entries(sizes[category]).map(([key, size]) => (
        <button
          key={key}
          onClick={() => handleSizeChange(size)}
          className="w-full text-left p-2 rounded-md hover:ring border-purple-400"
        >
          {size.name} ({size.width}x{size.height})
        </button>
      ))}
    </div>
  );

  useEffect(() => {
    if (typeof window !== "undefined") {
      // Get list of items
      let textc = localStorage.getItem("textc");
      let fontSize = localStorage.getItem("fontSize");
      let fontStyle = localStorage.getItem("fontStyle");
      if (!textc) {
        localStorage.setItem("textc", "#000000");
      }
      if (!fontSize) {
        localStorage.setItem("fontSize", 20);
      }
      if (!fontStyle) {
        localStorage.setItem("fontStyle", "Normal");
      }
    }
  }, []);
  // drawing edits states
  const [pencilColor, setPencilColor] = useState(
    (typeof window !== "undefined" && localStorage.getItem("pc")) || "#000000"
  );

  // text edits states
  const [textProps, settextProps] = useState({
    color:
      (typeof window !== "undefined" && localStorage.getItem("textc")) ||
      "#000000",
    fontSize:
      (typeof window !== "undefined" && localStorage.getItem("fontSize")) || 20,
    textSelectionIsActive: false,
  });
  const [shapesColor, setshapesColor] = useState({
    rectangleColor:
      (typeof window !== "undefined" &&
        localStorage.getItem("rectangleColor")) ||
      "#000000",
    circleColor:
      (typeof window !== "undefined" && localStorage.getItem("circleColor")) ||
      "#000000",
    triangleColor:
      (typeof window !== "undefined" &&
        localStorage.getItem("triangleColor")) ||
      "#000000",
  });
  const [selectedObject, setSelectedObject] = useState(null);
  const [zoomLevel, setZoomLevel] = useState(1);
  const [selectedShapeTool, setselectedShapeTool] = useState(null);
  const [showSizeCropModal, setShowSizeCropModal] = useState(false);
  const [customSize, setCustomSize] = useState({});
  // State variables for background fill
  const [fillColor, setFillColor] = useState("#ffffff");
  const [fillType, setFillType] = useState("solid"); // 'solid' or 'gradient'
  const [gradientStartColor, setGradientStartColor] = useState("#ffffff");
  const [gradientEndColor, setGradientEndColor] = useState("#000000");
  const [lastAction, setLastAction] = useState(null);
  const [lockImageLayer, setlockImageLayer] = useState(false);
  const isDeleting = useRef(false);
  const [selectedShape, setselectedShape] = useState({
    triangle: false,
    circle: false,
    rectangle: false,
  });
  // state variables for image filter
  const [isImage, setisImage] = useState(false);
  // state variables for layers
  const [layers, setLayers] = useState([]);
  const [selectedLayer, setSelectedLayer] = useState(null);
  const [isOpen, setIsOpen] = useState(false);
  const selectRef = useRef(null);
  const [borderCanvas, setborderCanvas] = useState(false); // Initial text color
  const [backgroundColor, setbackgroundColor] = useState(false);
  const [activeBlock, setactiveBlock] = useState({
    zoom: false,
    crop: false,
    shapes: false,
    brush: false,
    fill: false,
    pencil: false,
    text: false,
  });

   const toggleDropdown = () => {
    toggleModal();
    let subs = localStorage.getItem("subscribed");
    if (subs) {
      setIsOpen(!isOpen);
    } else {
      setTimeout(() => {
        setIsOpen(!isOpen);
      }, 5000);
    }
  };
  //  set border active to the canvas
  useEffect(() => {
    if (canvas) {
      if (selectedTool === "Select") {
        canvas.on("mouse:up", (options) => {
          if (backgroundColor) {
            setborderCanvas(true);
          } else {
            setborderCanvas(false);
          }
        });
      }
      // detect current objects
      canvas.on("mouse:up", () => {
        console.log("asdf");
        let currentActiveObject = canvas.getActiveObject();
        console.log("currentActiveObject", currentActiveObject);
        if (currentActiveObject?.type === "i-text") {
          setSelectedTool("Text");
        } else if (
          currentActiveObject?.type === "path" ||
          currentActiveObject?.type === "circle" ||
          currentActiveObject?.type === "polygon" ||
          currentActiveObject?.type === "rectangle" ||
          currentActiveObject?.type === "triangle"
        ) {
          setSelectedTool("Draw");
        } else if (
          currentActiveObject?.type === "triangle" ||
          currentActiveObject?.type === "circle" ||
          currentActiveObject?.type === "rectangle"
        ) {
          setSelectedTool("shapes");
        }
      });

      const textIsbeingSelected = (status) => {
        if (status === "on") {
          console.log("active");
          settextProps((prevProps) => ({
            ...prevProps,
            textSelectionIsActive: true,
          }));
        } else if (status === "off") {
          console.log("deactive");
          settextProps((prevProps) => ({
            ...prevProps,
            textSelectionIsActive: false,
          }));
        }
      };
      canvas.on("text:selection:changed", () => textIsbeingSelected("on"));
      canvas.on("selection:cleared", () => textIsbeingSelected("off"));
    }
  }, [selectedTool, backgroundColor, borderCanvas]);

  const handleDownload = (format) => {
    // Save current background
    canvas.setBackgroundColor(fillColor, canvas.renderAll.bind(canvas));
    exportCanvas(format);
    setIsOpen(false); // Close dropdown after selection
    window.location.reload();
  };
  // Function to export canvas data to an image blob
  const exportCanvas = (format) => {
    // Logic for high-resolution export
    let highResolution = true;
    let lowResolution = false;
    if (highResolution) {
      const exportWidth = canvas.width * 2; // Example: doubling the width for higher resolution
      const exportHeight = canvas.height * 2; // Example: doubling the height for higher resolution
      // Temporarily set canvas dimensions for high-resolution export
      canvas.setDimensions({ width: exportWidth, height: exportHeight });
    }

    if (lowResolution) {
      const exportWidth = canvas.width / 2; // Example: doubling the width for higher resolution
      const exportHeight = canvas.height / 2; // Example: doubling the height for higher resolution

      // Temporarily set canvas dimensions for high-resolution export
      canvas.setDimensions({ width: exportWidth, height: exportHeight });
    }
    // Choose format based on user selection or default
    format = format || "png";

    if (format === "pdf" || format === "svg") {
      // Handle PDF and SVG exports using Fabric.js methods
      // Example:
      const data = canvas.toSVG(); // For SVG export
      // const data = canvas.toDataURL('pdf'); // For PDF export (requires plugin or server-side processing)
      // Implement server-side logic for PDF generation if needed
      // Send data to server for PDF generation and download
    } else {
      // For image formats (jpg, png)
      const dataURL = canvas.toDataURL({ format: "image/" + format });
      downloadURI(dataURL, `drawing.${format}`);
    }
  };

  // Helper function to trigger download
  const downloadURI = (uri, name) => {
    const link = document.createElement("a");
    link.download = name;
    link.href = uri;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  };

  useEffect(() => {
    const initCanvas = () => {
      const canvas = new window.fabric.Canvas(canvasRef.current, {
        isDrawingMode: false,
        preserveObjectStacking: true,
        width: canvasSize.width,
        height: canvasSize.height,
      });

      window.fabric.Object.prototype.cornerColor = "white"; // Change corner color
      window.fabric.Object.prototype.borderColor = "#6D5ACF";
      fabric.Object.prototype.cornerStrokeColor = "#6D5ACF"; // Corner border color

      const dotSize = 2;
      const spacing = 20;
      const patternCanvas = document.createElement("canvas");
      patternCanvas.width = spacing;
      patternCanvas.height = spacing;

      const patternCtx = patternCanvas.getContext("2d");

      // Set background color (slight off-white for warmth)
      patternCtx.fillStyle = "#FCFCFC";
      patternCtx.fillRect(0, 0, spacing, spacing);

      // Draw more visible dot
      patternCtx.fillStyle = "rgba(0, 0, 0, 0.2)"; // Slightly darker for better visibility
      patternCtx.beginPath();
      patternCtx.arc(spacing / 2, spacing / 2, dotSize / 2, 0, Math.PI * 2);
      patternCtx.fill();

      const pattern = new fabric.Pattern({
        source: patternCanvas,
        repeat: "repeat",
      });

      canvas.setBackgroundColor(pattern, canvas.renderAll.bind(canvas));
      // delete icon
      const img = document.createElement("img");
      img.src = "delicon.svg";

      window.fabric.Object.prototype.transparentCorners = false;
      // window.fabric.Object.prototype.cornerColor = "blue";
      window.fabric.Object.prototype.cornerStyle = "circle";
      // 1.Delete control for window.Fabric objects
      window.fabric.Object.prototype.controls.deleteControl =
        new window.fabric.Control({
          x: 0.55,
          y: -0.6,
          offsetY: 16,
          cursorStyle: "pointer",
          mouseUpHandler: deleteObject,
          render: renderIconDelete,
          cornerSize: 30,
        });
      canvas.on("mouse:down", (e) => {
        const selectedObject = e.target;
        if (selectedObject && selectedObject.type === "image") {
          setisImage(selectedObject);
          setSelectedTool("image");
        }
      });

      canvas.on("object:added", (e) => {
        setLastAction(e.target);
      });
      canvas.on("object:modified", (e) => {
        setLastAction(e.target);
      });

      // Delete multiple selected objects function
      function deleteObject() {
        if (canvas) {
          isDeleting.current = true;
          const selectedObjects = canvas.getActiveObjects();

          if (selectedObjects.length > 0) {
            // Remove each selected object
            selectedObjects.forEach((obj) => {
              // Check if the object is an image
              if (obj.type === "image") {
                setisImage(null);
              }
              canvas.remove(obj);
            });

            // Clear the selection
            canvas.discardActiveObject();
            canvas.requestRenderAll();

            setTimeout(() => {
              isDeleting.current = false;
            }, 200); // small delay to ensure state is managed
          }
        }
      }

      // Render delete icon function
      function renderIconDelete(ctx, left, top, styleOverride, fabricObject) {
        const size = this.cornerSize;
        ctx.save();
        ctx.translate(left, top);
        ctx.rotate(window.fabric.util.degreesToRadians(fabricObject.angle));
        ctx.drawImage(img, -size / 2, -size / 2, size, size);
        ctx.restore();
      }
      canvas.getObjects().forEach((obj) => {
        obj.setControlsVisibility({
          mtr: false,
          deleteControl: true,
        });
      });

      return canvas;
    };
    const canvasElement = initCanvas();
    setCanvas(canvasElement);

    return () => {
      canvasElement.dispose(); // very important
    };
  }, []);

  const changefillcolor = (e) => {
    setFillType(e.target.value);
  };

  useEffect(() => {
    if (selectRef.current) {
      selectRef.current.selectedIndex = 0;
    }
  }, [isImage]);

  const resizeCanvas = (newSize) => {
    if (canvas) {
      setCanvasSize(newSize);
      // Check if the canvas already has a fill color
      const currentFill = canvas.backgroundColor;
      if (currentFill) {
        // Apply the fill color
        canvas.setBackgroundColor(currentFill, canvas.renderAll.bind(canvas));
      } else {
        // Add background image if no fill color is present
        const dotSize = 2;
        const spacing = 20;

        const patternCanvas = document.createElement("canvas");
        patternCanvas.width = spacing;
        patternCanvas.height = spacing;

        const patternCtx = patternCanvas.getContext("2d");

        // Set background color (slight off-white for warmth)
        patternCtx.fillStyle = "#FCFCFC";
        patternCtx.fillRect(0, 0, spacing, spacing);

        // Draw more visible dot
        patternCtx.fillStyle = "rgba(0, 0, 0, 0.2)"; // Slightly darker for better visibility
        patternCtx.beginPath();
        patternCtx.arc(spacing / 2, spacing / 2, dotSize / 2, 0, Math.PI * 2);
        patternCtx.fill();

        const pattern = new fabric.Pattern({
          source: patternCanvas,
          repeat: "repeat",
        });

        canvas.setBackgroundColor(pattern, canvas.renderAll.bind(canvas));
      }
    }
  };

  function handleDrawShapes(shape) {
    let path;
    setselectedShapeTool(shape);
    switch (shape) {
      case "Rectangle":
        drawRect();
        break;
      case "Circle":
        drawCircle();
        break;
      case "Triangle":
        drawTriangle();
        break;
      case "HalfCircle":
        // Define the half-circle path using an SVG path
        path = new fabric.Path("M 100 250 A 150 150 0 0 1 400 250", {
          fill: "", // No fill for the path
          stroke: "black",
          strokeWidth: 2,
        });
        break;
      case "zigzag":
        // Define zigzag path
        path = new fabric.Path(
          "M 10 250 L 60 200 L 110 250 L 160 200 L 210 250",
          {
            fill: "",
            stroke: "black",
            strokeWidth: 2,
            selectable: false,
          }
        );
        break;
      case "wave":
        // Define wave path
        path = new fabric.Path(
          "M 0 250 Q 50 150, 100 250 T 200 250 T 300 250",
          {
            fill: "",
            stroke: "black",
            strokeWidth: 2,
            selectable: false,
          }
        );
        break;
      default:
        break;
    }
  }
  // drawing shapes

  // 1. rectangle
  const drawRect = () => {
    setselectedShape({ rectangle: true, circle: false, triangle: false });
    canvas.isDrawingMode = false;
    clearCanvasListeners();
    let isDrawing = false;
    let startPosition = { x: 0, y: 0 };
    let rect = null;
    if (canvas) {
      canvas.on("mouse:down", (options) => {
        isDrawing = true;
        const pointer = canvas.getPointer(options.e);
        startPosition = { x: pointer.x, y: pointer.y };
        rect = new window.fabric.Rect({
          left: startPosition.x,
          top: startPosition.y,
          width: 0,
          height: 0,
          fill: "transparent",
          stroke: "black",
          strokeWidth: 0.4,
        });
        canvas.add(rect);
      });

      canvas.on("mouse:move", (options) => {
        if (!isDrawing) return;
        const pointer = canvas.getPointer(options.e);
        rect.set({
          width: pointer.x - startPosition.x,
          height: pointer.y - startPosition.y,
        });
        canvas.renderAll();
      });

      canvas.on("mouse:up", () => {
        isDrawing = false;
        rect.setCoords();
        rect = null;
        clearCanvasListeners();
      });
    }
  };
  // 2. circle
  const drawCircle = () => {
    setselectedShape({ rectangle: false, circle: true, triangle: false });
    canvas.isDrawingMode = false;
    clearCanvasListeners();
    if (canvas) {
      let isDrawing = false;
      let circle = null;
      let startPosition = { x: 0, y: 0 };

      canvas.on("mouse:down", (options) => {
        isDrawing = true;
        const pointer = canvas.getPointer(options.e);
        startPosition = { x: pointer.x, y: pointer.y };

        // Check if there's already a circle being drawn
        if (!circle) {
          circle = new window.fabric.Circle({
            left: startPosition.x,
            top: startPosition.y,
            radius: 0,
            fill: "transparent",
            stroke: "black",
            strokeWidth: 0.4,
          });
          canvas.add(circle);
        }
      });

      canvas.on("mouse:move", (options) => {
        if (!isDrawing) return;
        const pointer = canvas.getPointer(options.e);
        const radius = Math.abs(pointer.x - startPosition.x);
        circle.set({ radius });
        canvas.renderAll();
      });

      canvas.on("mouse:up", () => {
        isDrawing = false;
        circle.setCoords();
        circle = null; // Reset circle for next draw
        clearCanvasListeners();
      });
    }
  };

  const drawTriangle = () => {
    setselectedShape({ rectangle: false, circle: false, triangle: true });
    canvas.isDrawingMode = false;
    clearCanvasListeners();
    let isDrawing = false;
    let startPosition = { x: 0, y: 0 };
    let triangle = null;
    if (canvas) {
      canvas.on("mouse:down", (options) => {
        isDrawing = true;
        const pointer = canvas.getPointer(options.e);
        startPosition = { x: pointer.x, y: pointer.y };
        triangle = new window.fabric.Triangle({
          left: startPosition.x,
          top: startPosition.y,
          width: 0,
          height: 0,
          fill: "transparent",
          stroke: "black",
          strokeWidth: 0.4,
        });
        canvas.add(triangle);
      });

      canvas.on("mouse:move", (options) => {
        if (!isDrawing) return;
        const pointer = canvas.getPointer(options.e);
        const width = Math.abs(pointer.x - startPosition.x);
        const height = Math.abs(pointer.y - startPosition.y);
        triangle.set({
          width: width * 2,
          height: height,
          left: startPosition.x - width,
          top:
            pointer.y > startPosition.y
              ? startPosition.y
              : startPosition.y - height,
        });
        canvas.renderAll();
      });

      canvas.on("mouse:up", () => {
        isDrawing = false;
        triangle.setCoords();
        triangle = null; // Reset circle for next draw
        clearCanvasListeners();
      });
    }
  };

  // removeBackgroundColor
  const removeBackgroundColor = () => {
    if (canvas) {
      const dotSize = 2;
      const spacing = 20;
      const patternCanvas = document.createElement("canvas");
      patternCanvas.width = spacing;
      patternCanvas.height = spacing;

      const patternCtx = patternCanvas.getContext("2d");
      // Set background color (slight off-white for warmth)
      patternCtx.fillStyle = "#FCFCFC";
      patternCtx.fillRect(0, 0, spacing, spacing);

      // Draw more visible dot
      patternCtx.fillStyle = "rgba(0, 0, 0, 0.2)"; // Slightly darker for better visibility
      patternCtx.beginPath();
      patternCtx.arc(spacing / 2, spacing / 2, dotSize / 2, 0, Math.PI * 2);
      patternCtx.fill();

      const pattern = new fabric.Pattern({
        source: patternCanvas,
        repeat: "repeat",
      });

      canvas.setBackgroundColor(pattern, canvas.renderAll.bind(canvas));
      setborderCanvas(false);
    }
  };
  // Utility function to clear existing canvas listeners.
  // So we should clear the old attatched evnets for next process.
  const clearCanvasListeners = () => {
    if (canvas) {
      canvas.off("mouse:down");
      canvas.off("mouse:move");
      canvas.off("mouse:up");
    }
  };

  const writeText = () => {
    canvas.isDrawingMode = false;
    // Create a textbox with default content
    let textbox = new window.fabric.IText("Type your text here!", {
      fontSize: textProps.fontSize,
      fill: textProps.color,
      fontFamily: "Arial",
    });
    // Calculate the center position
    const canvasWidth = canvas.width;
    const canvasHeight = canvas.height;

    const textboxWidth = textbox.width * textbox.scaleX; // Get the actual width after scaling
    const textboxHeight = textbox.height * textbox.scaleY; // Get the actual height after scaling

    // Set the textbox position to center it
    textbox.left = (canvasWidth - textboxWidth) / 2;
    textbox.top = (canvasHeight - textboxHeight) / 2;

    function renderIconDelete(ctx, left, top, styleOverride, fabricObject) {
      const size = this.cornerSize;
      ctx.save();
      ctx.translate(left, top);
      ctx.rotate(window.fabric.util.degreesToRadians(fabricObject.angle));
      ctx.drawImage(img, -size / 2, -size / 2, size, size);
      ctx.restore();
    }

    const img = document.createElement("img");
    img.src = "delicon.svg";

    // Add custom delete control to the textbox
    textbox.controls.deleteControl = new fabric.Control({
      x: 0.5, // Position the control on the top-right
      y: -0.5,
      offsetX: 16, // Adjust position if needed
      offsetY: -16,
      cursorStyle: "pointer",
      mouseUpHandler: function (eventData, transform) {
        const target = transform.target;
        canvas.remove(target); // Remove the object on click
        canvas.requestRenderAll();
      },
      render: renderIconDelete,
      cornerSize: 24,
    });

    // Set control visibility
    textbox.setControlsVisibility({
      mtr: false, // Hide the rotation control
      deleteControl: true, // Show the delete control
    });

    // Add the textbox to the canvas and set it as active
    canvas.add(textbox).setActiveObject(textbox);
  };
  const handleSelectTool = (tool) => {
    setSelectedTool(tool); // please integrate one of that tool.
    switch (tool) {
      case "Select":
        clearCanvasListeners();
        setstartDrawing(false);
        canvas.isDrawingMode = false;
        clearCanvasListeners();
        break;
      case "Wipe":
        canvas.off("path:created");
        canvas.isDrawingMode = true; // Enable drawing mode
        canvas.freeDrawingBrush = new window.fabric.EraserBrush(canvas); // Set eraser brush
        canvas.freeDrawingBrush.width = WiperWidth;
        console.log("WiperWidth", WiperWidth);
        break;
      // we will add a pencil or other shapes we can draw
      case "zoom":
        clearCanvasListeners();
        setactiveBlock({ zoom: !activeBlock.zoom });
        canvas.isDrawingMode = false;
        break;
      case "Resize":
        clearCanvasListeners();
        setstartDrawing(false);
        setactiveBlock({ crop: !activeBlock.crop });
        break;
      case "Fill":
        clearCanvasListeners();
        setstartDrawing(false);
        if (canvas) {
          setactiveBlock({ fill: !activeBlock.fill });
        }

        break;
      case "Shapes":
        setstartDrawing(false);
        setactiveBlock({ shapes: !activeBlock.shapes });
        break;

      case "Image":
        clearCanvasListeners();
        setstartDrawing(false);
        canvas.isDrawingMode = false;
        handleAddImage();
        break;

      case "layers":
        setstartDrawing(false);
        canvas.isDrawingMode = false;
        getAllLayers();
        break;
      case "Draw":
        clearCanvasListeners();
        setactiveBlock({ pencil: !activeBlock.pencil });
        break;

      case "Text":
        clearCanvasListeners();
        setactiveBlock({ text: !activeBlock.text });
        writeText();
        break;

      case "Undo":
        clearCanvasListeners();
        handleUndo();
        break;
      default:
        canvas.isDrawingMode = false;
        clearCanvasListeners();
    }
  };

  const handleFontStyleChange = (e) => {
    const style = e.target.value;
    const activeObject = canvas.getActiveObject();

    if (activeObject) {
      let updates = {};
      switch (style) {
        case "Bold":
          updates.fontWeight = "bold";
          break;
        case "Italic":
          updates.fontStyle = "italic";
          break;
        case "Underline":
          updates.underline = true;
          break;
        case "StrikeThrough":
          updates.linethrough = true;
          break;
        case "Normal":
          updates = {
            fontWeight: "normal",
            fontStyle: "normal",
            underline: false,
            linethrough: false,
            fontVariant: "normal",
            fontStretch: "normal",
          };
          break;

        case "Uppercase":
          activeObject.set("text", activeObject.text.toUpperCase());
          break;
        case "Lowercase":
          activeObject.set("text", activeObject.text.toLowerCase());
          break;
        case "Capitalize":
          activeObject.set(
            "text",
            activeObject.text.replace(/\b\w/g, (char) => char.toUpperCase())
          );
          break;
        default:
          updates = {
            fontWeight: "normal",
            fontStyle: "normal",
            underline: false,
            linethrough: false,
            fontVariant: "normal",
            fontStretch: "normal",
          };
      }

      activeObject.set(updates);
      canvas.renderAll();
      // Update state and localStorage
      settextProps({ fontStyle: style });
      localStorage.setItem("fontStyle", style);
    }
  };
  //  ClearCanvas
  const ClearCanvas = () => {
    canvas.clear();
    const dotSize = 2;
    const spacing = 20;

    const patternCanvas = document.createElement("canvas");
    patternCanvas.width = spacing;
    patternCanvas.height = spacing;
    const patternCtx = patternCanvas.getContext("2d");

    // Set background color (slight off-white for warmth)
    patternCtx.fillStyle = "#FCFCFC";
    patternCtx.fillRect(0, 0, spacing, spacing);

    // Draw more visible dot
    patternCtx.fillStyle = "rgba(0, 0, 0, 0.2)"; // Slightly darker for better visibility
    patternCtx.beginPath();
    patternCtx.arc(spacing / 2, spacing / 2, dotSize / 2, 0, Math.PI * 2);
    patternCtx.fill();

    const pattern = new fabric.Pattern({
      source: patternCanvas,
      repeat: "repeat",
    });
    canvas.setBackgroundColor(pattern, canvas.renderAll.bind(canvas));
    const json = JSON.stringify(canvas.toJSON());
    localStorage.setItem("canvasState", json);
  };

  const handleTextSizeChange = (e) => {
    settextProps((prevState) => ({ ...prevState, fontSize: e.target.value }));
    localStorage.setItem("fontSize", e.target.value);
    const getActiveObject = canvas.getActiveObject();
    if (getActiveObject) {
      getActiveObject.set("fontSize", e.target.value);
    }
    canvas.renderAll();
  };

  const handleTextColorChange = (e) => {
    const activeObject = canvas.getActiveObject();
    let selectionStart;
    let selectionEnd;
    if (textProps.textSelectionIsActive) {
      if (activeObject) {
        selectionStart = activeObject?.selectionStart;
        selectionEnd = activeObject?.selectionEnd;
        localStorage.setItem("textc", e.target.value);
        settextProps((prevState) => ({ ...prevState, color: e.target.value }));
        // Apply styles to the selected text
        activeObject.setSelectionStyles(
          {
            fill: e.target.value,
            // fontSize: textProps.fontSize,
            // fontStyle: textProps.fontStyle,
          },
          selectionStart,
          selectionEnd
        );
      }
    } else {
      settextProps((prevState) => ({ ...prevState, color: e.target.value }));
      localStorage.setItem("textc", e.target.value);
      if (activeObject) {
        activeObject.set("styles", { fill: e.target.value });
        activeObject.set("fill", e.target.value);
      }
    }
    canvas.renderAll();
  };

  const handleAddImage = () => {
    if (isImage) return;
    const fileInput = document.createElement("input");
    fileInput.type = "file";
    fileInput.accept = "image/*";
    fileInput.onchange = (e) => {
      const file = e.target.files[0];
      const reader = new FileReader();
      reader.onload = (f) => {
        const data = f.target.result;
        window.fabric.Image.fromURL(data, (img) => {
          img.scaleToWidth(canvasSize.width / 2);
          img.scaleToHeight(canvasSize.height / 2);

          let imageTextureSize =
            img.width > img.height ? img.width : img.height;

          if (imageTextureSize > window.fabric.textureSize) {
            window.fabric.textureSize = imageTextureSize;
          }

          canvas.add(img);
          canvas.renderAll();
          setisImage(img);
        });
      };
      reader.readAsDataURL(file);
    };
    fileInput.click();
  };

  // Apply image filters
  const applyImageEffects = (selectedFilter) => {
    switch (selectedFilter) {
      case "reset":
        isImage.filters = [];
        isImage.applyFilters(); // Apply no filters
        canvas.renderAll(); // Render canvas
        return; // Exit function early for reset
      case "grayscale":
        isImage.filters = [];
        isImage.applyFilters();
        isImage.filters.push(new window.fabric.Image.filters.Grayscale());
        isImage.applyFilters();

        break;
      case "sepia":
        isImage.filters = [];
        isImage.applyFilters();
        isImage.filters.push(new window.fabric.Image.filters.Sepia());
        isImage.applyFilters();
        break;
      case "invert":
        isImage.filters = [];
        isImage.applyFilters();
        isImage.filters.push(new window.fabric.Image.filters.Invert());
        isImage.applyFilters();
        break;
      case "brightness":
        isImage.filters = [];
        isImage.applyFilters();
        isImage.filters.push(
          new window.fabric.Image.filters.Brightness({ brightness: 0.5 })
        );
        isImage.applyFilters();
        break;
      case "contrast":
        isImage.filters = [];
        isImage.applyFilters();
        isImage.filters.push(
          new window.fabric.Image.filters.Contrast({ contrast: 0.5 })
        );
        isImage.applyFilters();
        break;
      case "blur":
        isImage.filters = [];
        isImage.applyFilters();
        isImage.filters.push(
          new window.fabric.Image.filters.Blur({ blur: 0.5 })
        );
        isImage.applyFilters();
        break;
      // Add more filters as needed
      default:
        break;
    }
    isImage.applyFilters();
    canvas.renderAll();
  };

  const handleUndo = () => {
    // Perform the undo operation
    if (canvas) {
      if (lastAction) {
        canvas.remove(lastAction);
        setLastAction(null);
        canvas.renderAll();
      }
      // canvas.undo();
      // Ensure that the background image is set initially
      const dotSize = 2;
      const spacing = 20;

      const patternCanvas = document.createElement("canvas");
      patternCanvas.width = spacing;
      patternCanvas.height = spacing;
      const patternCtx = patternCanvas.getContext("2d");

      // Set background color (slight off-white for warmth)
      patternCtx.fillStyle = "#FCFCFC";
      patternCtx.fillRect(0, 0, spacing, spacing);

      // Draw more visible dot
      patternCtx.fillStyle = "rgba(0, 0, 0, 0.2)"; // Slightly darker for better visibility
      patternCtx.beginPath();
      patternCtx.arc(spacing / 2, spacing / 2, dotSize / 2, 0, Math.PI * 2);
      patternCtx.fill();

      const pattern = new fabric.Pattern({
        source: patternCanvas,
        repeat: "repeat",
      });

      canvas.setBackgroundColor(pattern, canvas.renderAll.bind(canvas));
    }
    setstartDrawing(false);
  };

  // const handleRedo = () => {
  // canvas.redo();
  // };

  const handleDeleteObject = () => {
    if (selectedObject) {
      canvas.remove(selectedObject);
      canvas.discardActiveObject();
      setSelectedObject(null);
      canvas.renderAll();
    }
  };
  //  handleDrawColor
  const handleDrawColor = (e) => {
    let color = e.target.value;
    setDrawColor(color);
    let activeObject = canvas.getActiveObject();
    if (activeObject) {
      activeObject.set(drawColorSelectType, color);
    }
    canvas.renderAll();
  };
  // handleRotateObject
  const handleRotateObject = () => {
    if (selectedObject) {
      selectedObject.rotate((selectedObject.angle + 45) % 360);
      canvas.renderAll();
    }
  };

  const handleZoom = (zoomIn) => {
    const zoomFactor = zoomIn ? 1.2 : 1 / 1.2; // Increased zoom factor for faster zoom
    const targetZoomLevel = zoomLevel * zoomFactor;

    // Get current viewport center (the visible part of the canvas)
    const canvasCenter = {
      x: (canvas.width / 2 - canvas.viewportTransform[4]) / canvas.getZoom(),
      y: (canvas.height / 2 - canvas.viewportTransform[5]) / canvas.getZoom(),
    };

    // Define center point based on current viewport center
    const centerPoint = new fabric.Point(canvasCenter.x, canvasCenter.y);

    const animateZoom = () => {
      const currentZoom = canvas.getZoom();
      const zoomStep = (targetZoomLevel - currentZoom) * 0.2; // Increased zoom step for faster zoom

      if (Math.abs(zoomStep) > 0.001) {
        canvas.zoomToPoint(centerPoint, currentZoom + zoomStep);

        requestAnimationFrame(animateZoom);
      } else {
        // Final zoom level
        canvas.zoomToPoint(centerPoint, targetZoomLevel);
        setZoomLevel(targetZoomLevel);
      }

      canvas.requestRenderAll(); // Ensure smooth rendering
    };

    requestAnimationFrame(animateZoom);
  };

  const handleSizeCropSubmit = () => {
    resizeCanvas(customSize);
    setShowSizeCropModal(false);
  };

  const handleDefaultSize = () => {
    resizeCanvas({
      width: window.innerWidth - 256,
      height: window.innerHeight,
    });
    setShowSizeCropModal(false);
  };

  const handleBackgroundFill = () => {
    if (fillType === "solid") {
      canvas.setBackgroundImage(null); // Remove background image
      canvas.setBackgroundColor(fillColor, canvas.renderAll.bind(canvas));
      setbackgroundColor(true);
      // set the whole canvas background
    } else if (fillType === "gradient") {
      canvas.setBackgroundImage(null); // Remove background image
      setbackgroundColor(true);
      // Define the gradient object
      let gradient = new window.fabric.Gradient({
        type: "linear",
        coords: {
          x1: canvas.width / 2,
          y1: canvas.height / 2,
          // x2: canvas.width / 2, // these types will be for the radial
          // y2: canvas.height / 2,
          // r1: 10, // inner circle radius
          // r2: canvas.width / 2, // outer circle radius
        },
        colorStops: [
          { offset: 0, color: gradientStartColor },
          { offset: 0.2, color: gradientEndColor },
        ],
      });
      canvas.set("backgroundColor", gradient);
      canvas.renderAll();
    }
  };

  // get layers
  const addLayer = () => {
    const newLayer = new window.fabric.Rect({
      width: 500,
      height: 500,
      fill: "rgba(0, 0, 0, 0)",
      selectable: true,
      left: canvasSize.width / 2 - 50, // Center horizontally
      top: canvasSize.height / 2 - 50, // Center vertically
    });
    canvas.add(newLayer); // first add a new layer to canvas
    canvas.setActiveObject(newLayer); // then active the controls selection
    setLayers([...layers, newLayer]);
  };

  // layer management functions
  const getAllLayers = () => {
    if (canvas) {
      setLayers(canvas.getObjects());
    }
  };
  // selectLayer
  const selectLayer = (layer) => {
    if (canvas) {
      setSelectedLayer(layer);
      canvas.setActiveObject(layer);
      canvas.renderAll();
    }
  };
  const deleteLayer = () => {
    if (selectedLayer) {
      canvas.remove(selectedLayer);
      setLayers(layers.filter((layer) => layer !== selectedLayer));
      setSelectedLayer(null);
      canvas.renderAll();
    }
  };

  const moveLayerUp = () => {
    if (selectedLayer) {
      const index = layers.indexOf(selectedLayer);
      if (index !== -1 && index < layers.length - 1) {
        canvas.moveTo(selectedLayer, index + 1);
        setLayers([
          ...layers.slice(0, index),
          layers[index + 1],
          layers[index],
          ...layers.slice(index + 2),
        ]);
        setSelectedLayer(layers[index + 1]);
      }
    }
  };

  const moveLayerDown = () => {
    if (selectedLayer) {
      const index = layers.indexOf(selectedLayer);
      if (index > 0) {
        canvas.moveTo(selectedLayer, index - 1);
        setLayers([
          ...layers.slice(0, index - 1),
          layers[index],
          layers[index - 1],
          ...layers.slice(index + 1),
        ]);
        setSelectedLayer(layers[index - 1]);
      }
    }
  };

  const toggleLayerVisibility = () => {
    if (selectedLayer) {
      selectedLayer.visible = !selectedLayer.visible;
      canvas.renderAll();
    }
  };

  const toggleLayerLock = () => {
    if (selectedLayer) {
      selectedLayer.lockMovementX = !selectedLayer.lockMovementX;
      selectedLayer.lockMovementY = !selectedLayer.lockMovementY;
      selectedLayer.lockScalingX = !selectedLayer.lockScalingX;
      selectedLayer.lockScalingY = !selectedLayer.lockScalingY;
      selectedLayer.lockRotation = !selectedLayer.lockRotation;
      canvas.renderAll();
    }
  };

  const setImageLayerFn = (e) => {
    if (lockImageLayer) {
      setlockImageLayer(false);
      localStorage.setItem("lockimg", false);
      // unlock the image
      isImage.set({
        selectable: true,
        evented: true,
        lockMovementX: false,
        lockMovementY: false,
        lockRotation: false,
        lockScalingX: false,
        lockScalingY: false,
      });
      canvas.renderAll();
    } else if (!lockImageLayer) {
      setlockImageLayer(true);
      localStorage.setItem("lockimg", true);
      if (isImage) {
        isImage.set({
          selectable: false,
          evented: false,
          lockMovementX: true,
          lockMovementY: true,
          lockRotation: true,
          lockScalingX: true,
          lockScalingY: true,
        });
        setlockImageLayer(true);
        canvas.renderAll();
      }
    }
  };

  let zoomIn = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      fill="none"
      viewBox="0 0 24 24"
      strokeWidth="1.5"
      stroke="currentColor"
      className="size-5"
    >
      <path
        strokeLinecap="round"
        strokeLinejoin="round"
        d="m21 21-5.197-5.197m0 0A7.5 7.5 0 1 0 5.196 5.196a7.5 7.5 0 0 0 10.607 10.607ZM10.5 7.5v6m3-3h-6"
      />
    </svg>
  );
  let zoomout = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      fill="none"
      viewBox="0 0 24 24"
      strokeWidth="1.5"
      stroke="currentColor"
      className="size-5"
    >
      <path
        strokeLinecap="round"
        strokeLinejoin="round"
        d="m21 21-5.197-5.197m0 0A7.5 7.5 0 1 0 5.196 5.196a7.5 7.5 0 0 0 10.607 10.607ZM13.5 10.5h-6"
      />
    </svg>
  );

  // icons list
  let cursorIcon = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      fill="none"
      viewBox="0 0 14 14"
      id="Arrow-Cursor-2--Streamline-Core"
      height={14}
      width={14}
      className="size-5"
    >
      <desc>{"Arrow Cursor 2 Streamline Icon: https://streamlinehq.com"}</desc>
      <g id="arrow-cursor-2--mouse-select-cursor">
        <path
          id="Vector"
          stroke="#000000"
          strokeLinecap="round"
          strokeLinejoin="round"
          d="M13.0055 5.42823c0.1561 -0.06508 0.2569 -0.09398 0.3497 -0.18665 0.0927 -0.09268 0.1448 -0.21836 0.1448 -0.34942s-0.0521 -0.25675 -0.1448 -0.34942c-0.0928 -0.09267 -0.1241 -0.14819 -0.3497 -0.21437C12.7798 4.26219 1.80942 0.553527 1.80942 0.553527c-0.17507 -0.059892 -0.36344 -0.069509 -0.5437 -0.027758C1.08546 0.567521 0.920531 0.658965 0.789686 0.78971c-0.130844 0.130745 -0.222358 0.29555 -0.264142 0.47567 -0.041783 0.18013 -0.032159 0.36835 0.027779 0.54329 0 0 3.796377 11.04723 3.842757 11.19743 0.04638 0.1502 0.10621 0.2567 0.19895 0.3494 0.09274 0.0927 0.21852 0.1447 0.34968 0.1447s0.25694 -0.052 0.34968 -0.1447c0.09274 -0.0927 0.11394 -0.1452 0.18346 -0.3494 0.06953 -0.2042 1.93949 -5.59378 1.93949 -5.59378s5.43206 -1.91901 5.58816 -1.98409Z"
          strokeWidth={1}
        />
      </g>
    </svg>
  );

  let crop = (
    <svg
      viewBox="0 0 16 16"
      fill="none"
      xmlns="http://www.w3.org/2000/svg"
      id="Crop--Streamline-Radix"
      height={16}
      width={16}
      className="size-5"
    >
      <desc>{"Crop Streamline Icon: https://streamlinehq.com"}</desc>
      <path
        fillRule="evenodd"
        clipRule="evenodd"
        d="M3.7333333333333334 8.5408e-7C4.027882666666667 8.4128e-7 4.266666666666667 0.23878293333333334 4.266666666666667 0.5333344L4.266666666666667 3.200064l8 0c0.2945066666666667 0 0.5333333333333333 0.238784 0.5333333333333333 0.5333333333333333l0 8.000042666666667L15.466666666666667 11.733333333333333c0.2945066666666667 0 0.5333333333333333 0.23872 0.5333333333333333 0.5333333333333333 0 0.2945066666666667 -0.23872 0.5333333333333333 -0.5333333333333333 0.5333333333333333l-2.6666666666666665 0.00010666666666666667L12.8 15.466666666666667c0 0.2945066666666667 -0.23882666666666666 0.5333333333333333 -0.5333333333333333 0.5333333333333333s-0.5333333333333333 -0.23882666666666666 -0.5333333333333333 -0.5333333333333333l0 -2.6665599999999996 -8 0c-0.29454933333333333 0 -0.5333333333333333 -0.23882666666666666 -0.5333333333333333 -0.5333333333333333l0 -8.000053333333334L0.5333216000000001 4.266666666666667C0.23877013333333333 4.266666666666667 -0.00000651242666666667 4.027872 -5.3610666666667e-7 3.7333226666666666 0.00000547210666666667 3.4387733333333332 0.23879146666666667 3.2 0.5333439999999999 3.2L3.2 3.200053333333333 3.2 0.5333344C3.2 0.23878293333333334 3.438784 8.6698666666667e-7 3.7333333333333334 8.5408e-7ZM4.266666666666667 4.266730666666667l0 7.466709333333333 7.466666666666667 0 0 -7.466709333333333 -7.466666666666667 0Z"
        fill="currentColor"
        strokeWidth={1}
      />
    </svg>
  );

  let fill = (
    <svg
      className="size-5"
      viewBox="-0.5 -0.5 16 16"
      fill="currentColor"
      xmlns="http://www.w3.org/2000/svg"
      height="16"
      width="16"
      id="Paint-Fill--Streamline-Remix-Fill"
    >
      <path
        d="M12.0173125 11.707687499999999L13.122187499999999 10.6028125L14.227 11.707687499999999C14.837187499999999 12.317875 14.837187499999999 13.3071875 14.227 13.917375C13.6168125 14.5275625 12.627500000000001 14.5275625 12.0173125 13.917375C11.407125 13.3071875 11.407125 12.317875 12.0173125 11.707687499999999ZM5.54913125 0.67481875L12.6201875 7.745875C12.864249999999998 7.9899375 12.864249999999998 8.3856875 12.6201875 8.629750000000001L7.3168750000000005 13.9330625C7.0728124999999995 14.177125 6.6770625 14.177125 6.433 13.9330625L1.1297125 8.629750000000001C0.8856375 8.3856875 0.8856375 7.9899375 1.1297125 7.745875L5.991075 2.8845312499999998L4.66525 1.5587062500000002L5.54913125 0.67481875ZM6.8749375 3.7684125L2.4555374999999997 8.1878125H11.294375L6.8749375 3.7684125Z"
        stroke-width="1"
      ></path>
    </svg>
  );
  let image = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      className="size-5"
      viewBox="-0.5 -0.5 16 16"
      id="Pic-Fill--Streamline-Mingcute-Fill"
      height={16}
      width={16}
    >
      <desc>{"Pic Fill Streamline Icon: https://streamlinehq.com"}</desc>
      <g fill="none" fillRule="nonzero">
        <path
          d="M15 0v15H0V0h15ZM7.870625 14.536249999999999l-0.006874999999999999 0.00125 -0.044375 0.021875000000000002 -0.0125 0.0025 -0.00875 -0.0025 -0.044375 -0.021875000000000002c-0.00625 -0.0025 -0.011875 -0.000625 -0.015 0.003125l-0.0025 0.00625 -0.010625 0.2675 0.003125 0.0125 0.00625 0.008125 0.065 0.04625 0.009375 0.0025 0.0075 -0.0025 0.065 -0.04625 0.0075 -0.01 0.0025 -0.010625 -0.010625 -0.266875c-0.00125 -0.00625 -0.005625 -0.010625 -0.010625 -0.01125Zm0.16562500000000002 -0.07062500000000001 -0.008125 0.00125 -0.115625 0.058124999999999996 -0.00625 0.00625 -0.001875 0.006874999999999999 0.01125 0.26875 0.003125 0.0075 0.005 0.004375 0.12562500000000001 0.058124999999999996c0.0075 0.0025 0.014374999999999999 0 0.018125000000000002 -0.005l0.0025 -0.00875 -0.02125 -0.38375c-0.001875 -0.0075 -0.00625 -0.0125 -0.0125 -0.013749999999999998Zm-0.44687499999999997 0.00125a0.014374999999999999 0.014374999999999999 0 0 0 -0.016875 0.00375l-0.00375 0.00875 -0.02125 0.38375c0 0.0075 0.004375 0.0125 0.010625 0.015l0.009375 -0.00125 0.12562500000000001 -0.058124999999999996 0.00625 -0.005 0.0025 -0.006874999999999999 0.010625 -0.26875 -0.001875 -0.0075 -0.00625 -0.00625 -0.11499999999999999 -0.057499999999999996Z"
          strokeWidth={1}
        />
        <path
          fill="#000000"
          d="M12.5 1.875a1.25 1.25 0 0 1 1.25 1.25v8.75a1.25 1.25 0 0 1 -1.25 1.25H2.5a1.25 1.25 0 0 1 -1.25 -1.25V3.125a1.25 1.25 0 0 1 1.25 -1.25h10Zm0 1.25H2.5v6.3125l3.121875 -3.12125a0.78125 0.78125 0 0 1 1.105 0l2.5406250000000004 2.54125 0.7737499999999999 -0.7737499999999999a0.78125 0.78125 0 0 1 1.105 0L12.5 9.438125000000001V3.125Zm-2.8125 1.25a0.9375 0.9375 0 1 1 0 1.875 0.9375 0.9375 0 0 1 0 -1.875Z"
          strokeWidth={1}
        />
      </g>
    </svg>
  );

  // let layer = (
  //   <svg
  //     xmlns="http://www.w3.org/2000/svg"
  //     fill="none"
  //     viewBox="0 0 24 24"
  //     strokeWidth="1.5"
  //     stroke="currentColor"
  //     className="size-5"
  //   >
  //     <path
  //       strokeLinecap="round"
  //       strokeLinejoin="round"
  //       d="M12 4.5v15m7.5-7.5h-15"
  //     />
  //   </svg>
  // );

  let undo = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      fill="none"
      viewBox="0 0 24 24"
      strokeWidth="1.5"
      stroke="currentColor"
      className="size-5"
    >
      <path
        strokeLinecap="round"
        strokeLinejoin="round"
        d="m15 15-6 6m0 0-6-6m6 6V9a6 6 0 0 1 12 0v3"
      />
    </svg>
  );
  let pencil = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      className="size-5"
      fill="none"
      viewBox="0 0 24 24"
      id="Edit-Pencil--Streamline-Freehand"
      height={24}
      width={24}
    >
      <path
        fill="#000000"
        fillRule="evenodd"
        d="M23.9565 3.23342c-0.1294 -0.48682 -0.3729 -0.93581 -0.7103 -1.30979 -0.3375 -0.37398 -0.7592 -0.66218 -1.2302 -0.84073 -0.8893 -0.388205 -1.8914 -0.431 -2.8107 -0.120029 -0.3761 0.156479 -0.7164 0.387899 -1.0002 0.680169 -0.4177 0.41195 -0.7566 0.89668 -1.0002 1.43035 -0.1372 0.26539 -0.2381 0.54798 -0.3001 0.8402 -0.0223 0.08352 -0.012 0.17241 0.0289 0.24859s0.1092 0.13393 0.1911 0.16151c0.0374 0.01431 0.0773 0.02099 0.1173 0.01965 0.04 -0.00134 0.0794 -0.01067 0.1157 -0.02745 0.0364 -0.01678 0.069 -0.04066 0.096 -0.07024 0.027 -0.02958 0.0478 -0.06427 0.0611 -0.102l0.3101 -0.73018c0.2023 -0.36669 0.4548 -0.70335 0.7502 -1.00025 0.2688 -0.25144 0.595 -0.43342 0.9502 -0.53012 0.6769 -0.14678 1.3837 -0.05489 2.0005 0.26006 0.32 0.09829 0.6134 0.26831 0.8578 0.49711 0.2444 0.2288 0.4334 0.51035 0.5526 0.82321 0.1505 0.51318 0.1779 1.05463 0.08 1.58038 -0.1036 0.53125 -0.2786 1.04604 -0.5202 1.53038l-0.13 -0.07002c-1.0602 -0.54013 -3.5809 -1.81044 -4.351 -2.24054 -0.0811 -0.04291 -0.1758 -0.05228 -0.2637 -0.02609 -0.0879 0.02619 -0.1621 0.08584 -0.2065 0.16612 -0.0429 0.08108 -0.0522 0.17576 -0.026 0.26368 0.0261 0.08792 0.0858 0.16204 0.1661 0.20643l4.4411 2.42059c-0.0901 0.20005 -0.2401 0.50012 -0.4301 0.87021 -1.2203 2.42058 -4.1411 7.86188 -5.1813 9.67238 -0.4224 -0.4187 -0.8773 -0.8034 -1.3603 -1.1503 -0.2401 -0.16 -0.5002 -0.3201 -0.7502 -0.4601 -0.2615 -0.1385 -0.5322 -0.2588 -0.8102 -0.3601 -0.5124 -0.1896 -1.0415 -0.3303 -1.5804 -0.4201 0.0309 -0.1229 0.0711 -0.2432 0.12 -0.3601 0.9202 -2.1305 3.3208 -6.83165 4.4211 -9.00218 0.2501 -0.48012 0.4301 -0.8302 0.5101 -1.00025 0.2001 -0.38009 -0.4101 -0.48011 -0.4201 -0.47011 -0.2527 0.36727 -0.4735 0.75544 -0.6601 1.16028 -1.2103 2.19054 -3.811 6.79166 -4.8012 8.89216 -0.2301 0.4006 -0.3796 0.8423 -0.4401 1.3003 0.07 0.4201 0.5101 3.3908 0.8002 4.5811 0.0525 0.318 0.2029 0.6117 0.4301 0.8402 0.182 0.1046 0.3907 0.1532 0.6001 0.1401 0.4617 -0.0569 0.9111 -0.1887 1.3304 -0.3901 1.1145 -0.5155 2.1488 -1.1893 3.0707 -2.0005 0.2501 -0.2801 3.2608 -6.0715 4.9112 -9.47231 0.4901 -1.00024 0.8602 -1.79044 1.0002 -2.17053 0.3643 -0.75412 0.6751 -1.53289 0.9303 -2.33057 0.1812 -0.6266 0.2289 -1.28427 0.14 -1.93047ZM12.8238 20.3476h-0.15s-0.6802 -3.3008 -0.8402 -4.191c0.4701 0.23 1.0002 0.5401 1.5103 0.8402 0.3101 0.18 0.6402 0.3401 1.0003 0.5101 0.3601 0.17 0.6701 0.3101 1.0002 0.4801l0.7802 0.4701c-0.5664 0.4689 -1.1798 0.8779 -1.8304 1.2203 -0.465 0.275 -0.9579 0.4996 -1.4704 0.6702Z"
        clipRule="evenodd"
        strokeWidth={1}
      />
      <path
        fill="#000000"
        fillRule="evenodd"
        d="M12.0136 22.5181c-0.6131 -0.1299 -1.2177 -0.2969 -1.8105 -0.5001 -0.37266 -0.126 -0.73133 -0.2903 -1.07022 -0.4901 -0.28233 -0.1619 -0.49045 -0.4274 -0.58014 -0.7402 -0.02845 -0.442 0.0363 -0.885 0.19005 -1.3003 0.20368 -0.6137 0.20368 -1.2768 0 -1.8905 -0.16948 -0.3505 -0.44628 -0.6378 -0.79019 -0.8202 -0.42163 -0.2361 -0.88884 -0.3793 -1.37034 -0.4201 -0.86214 -0.0148 -1.72354 0.0589 -2.57062 0.2201 -1.25979 0.2561 -2.50248 0.5902 -3.720907 1.0002 -0.045871 0.0029 -0.090531 0.0159 -0.130748 0.0381 -0.040218 0.0223 -0.0749867 0.0532 -0.1017912 0.0905 -0.0268045 0.0374 -0.0449741 0.0802 -0.05319421 0.1254 -0.00822007 0.0452 -0.00628494 0.0917 0.00566531 0.1361 0.0093189 0.0391 0.0263803 0.0758 0.0501736 0.1082 0.0237933 0.0323 0.0538345 0.0596 0.0883445 0.0801 0.034511 0.0205 0.072788 0.0339 0.112565 0.0393 0.039777 0.0055 0.080245 0.0029 0.119007 -0.0076 1.000245 -0.21 2.000485 -0.4201 3.090755 -0.5601 0.99365 -0.1441 2.0008 -0.171 3.00073 -0.08 0.24742 0.0268 0.48799 0.0979 0.71017 0.21 0.18004 0.0901 0.37009 0.1801 0.43011 0.3301 0.12285 0.3907 0.12285 0.8097 0 1.2003 -0.16979 0.5818 -0.19723 1.1959 -0.08002 1.7905 0.08373 0.277 0.2238 0.5337 0.41141 0.754 0.18762 0.2203 0.41873 0.3995 0.67885 0.5263 0.41775 0.2103 0.8571 0.3746 1.31032 0.4901 0.63872 0.1676 1.28662 0.2978 1.94052 0.3901 0.09 0.0146 0.1822 -0.0065 0.2569 -0.0588 0.0747 -0.0523 0.1261 -0.1317 0.1431 -0.2213 0.0135 -0.046 0.0174 -0.0943 0.0114 -0.1418 -0.0059 -0.0476 -0.0216 -0.0934 -0.0459 -0.1346 -0.0244 -0.0413 -0.057 -0.0771 -0.0958 -0.1053 -0.0388 -0.0281 -0.0829 -0.048 -0.1297 -0.0584Z"
        clipRule="evenodd"
        strokeWidth={1}
      />
    </svg>
  );
  let textType = (
    <svg
      viewBox="0 0 16 16"
      fill="none"
      xmlns="http://www.w3.org/2000/svg"
      id="Text--Streamline-Radix"
      height={16}
      width={16}
      className="size-5"
    >
      <desc>{"Text Streamline Icon: https://streamlinehq.com"}</desc>
      <path
        fillRule="evenodd"
        clipRule="evenodd"
        d="M2.8932266666666666 1.4547199999999998V3.6843733333333333C2.8932266666666666 4.041813333333334 2.6034133333333336 4.331733333333333 2.2458666666666667 4.331733333333333C1.88832 4.331733333333333 1.5985066666666665 4.041813333333334 1.5985066666666665 3.6843733333333333V0.80736C1.5985066666666665 0.73888 1.6091733333333333 0.67296 1.6287999999999998 0.6110933333333333C1.712 0.34944 1.9568 0.16 2.2458666666666667 0.16H13.754133333333334C13.977706666666668 0.16 14.174613333333333 0.2731733333333333 14.290986666666667 0.44544C14.360746666666667 0.5486933333333333 14.401493333333333 0.67328 14.401493333333333 0.80736V0.80736V3.6843733333333333C14.401493333333333 4.041813333333334 14.111786666666665 4.331733333333333 13.754133333333334 4.331733333333333C13.396693333333333 4.331733333333333 13.106773333333333 4.041813333333334 13.106773333333333 3.6843733333333333V1.4547199999999998H8.791253333333334V14.54528H10.523733333333332C10.88128 14.54528 11.171093333333332 14.835199999999999 11.171093333333332 15.192639999999999C11.171093333333332 15.550293333333334 10.88128 15.84 10.523733333333332 15.84H5.488853333333334C5.131306666666666 15.84 4.841493333333333 15.550293333333334 4.841493333333333 15.192639999999999C4.841493333333333 14.835199999999999 5.131306666666666 14.54528 5.488853333333334 14.54528H7.208853333333334V1.4547199999999998H2.8932266666666666Z"
        fill="#000000"
        strokeWidth={1}
      />
    </svg>
  );
  let shapes = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      viewBox="-0.5 -0.5 16 16"
      fill="currentColor"
      id="Shapes-Thin--Streamline-Phosphor-Thin"
      height="16"
      width="16"
      className="size-5"
    >
      <desc>Shapes Thin Streamline Icon: https://streamlinehq.com</desc>
      <path
        d="M3.52712109375 3.453134765625c-0.0695859375 -0.206197265625 -0.33629296875 -0.259740234375 -0.48007031249999993 -0.096380859375 -0.024755859374999998 0.028124999999999997 -0.043669921874999997 0.060884765625 -0.055646484375 0.096380859375L0.16439648437499998 11.9341640625c-0.060673828125 0.18281250000000002 0.075240234375 0.371443359375 0.26785546875 0.37175390625h5.654021484375c0.192615234375 -0.000310546875 0.32852929687500004 -0.18894140625 0.26785546875 -0.37175390625ZM0.824501953125 11.74051171875 3.259259765625 4.4362265625 5.6940234375 11.74051171875ZM13.15379296875 4.3902890625c0 -2.6114765625 -2.8270078124999998 -4.24365234375 -5.088615234374999 -2.9379140625 -2.261607421875 1.30573828125 -2.261607421875 4.57008984375 0 5.875828125 0.515712890625 0.297744140625 1.1007128906249999 0.4545 1.69620703125 0.4545 1.872767578125 -0.0019511718750000002 3.390462890625 -1.5196464843750002 3.392408203125 -3.3924140625Zm-6.219416015625 0c0 -2.17623046875 2.3558378906249997 -3.53637890625 4.24051171875 -2.4482636718750004 1.8846738281250002 1.0881210937499999 1.8846738281250002 3.808412109375 0 4.896527343750001 -0.42976171875 0.24812109375000002 -0.9172617187500001 0.378744140625 -1.4135039062499999 0.378744140625 -1.5613652343750002 0.0000703125 -2.8270078124999998 -1.265642578125 -2.8270078124999998 -2.8270078124999998Zm7.632919921875 5.088615234374999H8.347880859375c-0.156134765625 0 -0.282703125 0.126568359375 -0.282703125 0.282703125v3.957814453125c0 0.15612890625 0.126568359375 0.28269726562499997 0.282703125 0.28269726562499997h6.219416015625c0.156134765625 0 0.282703125 -0.126568359375 0.282703125 -0.28269726562499997v-3.957814453125c0 -0.156134765625 -0.126568359375 -0.282703125 -0.282703125 -0.282703125Zm-0.28269726562499997 3.957814453125h-5.654021484375v-3.3924140625h5.654021484375Z"
        stroke-width="1"
      ></path>
    </svg>
  );
  let wipeIcon = (
    <svg
      xmlns="http://www.w3.org/2000/svg"
      fill="none"
      viewBox="0 0 48 48"
      id="Eraser--Streamline-Plump"
      height="48"
      width="48"
      className="w-8 h-8"
    >
      <desc>Eraser Streamline Icon: https://streamlinehq.com</desc>
      <g id="eraser--text-remove-format-formatting-eraser-delete">
        <path
          id="Subtract"
          stroke="#000000"
          stroke-linecap="round"
          stroke-linejoin="round"
          d="M2.8775 25.2196C1.7313 26.7131 1.7815 28.6103 2.9821 30.0619C4.2424 31.5868 6.3341 33.9044 9.7331 37.201C9.935 37.3976 10.1337 37.59 10.3293 37.7772C10.9361 38.3561 11.7502 38.6666 12.5883 38.639C15.4446 38.5554 20.2231 38.4717 27.7983 38.4717C28.3254 38.4717 28.8337 38.2782 29.2238 37.9237C30.7258 36.5564 32.2052 35.1646 33.6613 33.7486C39.3947 28.1878 42.4675 24.6141 44.076 22.5182C45.2223 21.0257 45.171 19.1275 43.9714 17.6758C42.7111 16.151 40.6184 13.8334 37.2204 10.5368C33.8224 7.2403 31.4326 5.2103 29.8617 3.9897C28.3651 2.8247 26.4083 2.7755 24.8688 3.8873C22.7081 5.4466 19.0245 8.4284 13.2912 13.9892C7.5588 19.55 4.485 23.1237 2.8775 25.2196Z"
          stroke-width="3"
        ></path>
        <path
          id="Vector 1291"
          stroke="#000000"
          stroke-linecap="round"
          stroke-linejoin="round"
          d="M8.3003 44.9163H45.9512"
          stroke-width="3"
        ></path>
      </g>
    </svg>
  );
  let tools = [
    { name: "Select", icon: cursorIcon },
    { name: "Resize", icon: crop },
    { name: "Wipe", icon: wipeIcon },
    { name: "Draw", icon: pencil },
    { name: "Text", icon: textType },
    { name: "Shapes", icon: shapes },
    { name: "Fill", icon: fill },
    { name: "Image", icon: image },
    // { name: "layers", icon: layer },
    { name: "Undo", icon: undo },
    // Just for to see the last item clear
    { name: "", icon: "" },
  ];

  return (
    <>
      <CollectEmails showModal={showModal} onClose={toggleModal} />

      {/* Auto save Drawings */}
      {canvas && <AutoSaveDrawing canvas={canvas} />}
      {/* edit toolbar */}
      <div className="fixed w-full bg-gradient-to-r from-purple-400 via-purple-500 to-purple-500 text-white p-2 shadow-sm z-50">
        <div className="container mx-auto flex items-center justify-between">
          {/* <Link
            href={"/"}
            className="text-white hover:text-purple-200 transition-colors ml-2"
          >
            Home
          </Link> */}
          <div className="flex flex-col items-center">
            <h1 className="text-lg font-semibold text-white">WipeBoard</h1>
            <p className="text-xs text-purple-200">Free V.</p>
          </div>
        </div>
      </div>
      <div className="relative z-50 flex justify-center items-center">
        {/* 1. Edits for Drawing */}
        {/* edit colors */}
        {selectedTool === "Draw" && (
          <>
            <MultipleDrawingTools
              canvas={canvas}
              clearCanvasListeners={clearCanvasListeners}
            />
            <div className="absolute top-16 space-x-2 bg-white shadow-xl p-3 rounded-lg flex justify-center items-center">
              <input
                className="text-black rounded-color-input"
                type="color"
                value={DrawColor}
                onChange={(e) => handleDrawColor(e)}
              />
              <select
                onChange={(e) => setdrawColorSelectType(e.target.value)}
                className="p-1 rounded-lg text-xs border border-purple-500"
                defaultValue={"stroke"}
              >
                <option value={"stroke"}>Border</option>
                <option value={"fill"}>Fill</option>
              </select>
              <input
                type="range"
                min="0"
                max="15"
                // value={value}
                onChange={changeStrokeSize}
                style={{
                  width: "30px",
                  background: "#ddd", // Background color of the track
                  height: "5px", // Height of the track
                  borderRadius: "5px", // Round edges for the track
                  outline: "none", // Remove outline
                }}
              />
            </div>
          </>
        )}
        {/* Wiper */}
        {selectedTool === "Wipe" && (
          <>
            <div className="absolute top-16 space-x-2 bg-white shadow-xl p-3 rounded-lg flex justify-center items-center">
              <label htmlFor="wiperwidth">Wiper Size</label>
              <input
                id="wiperwidth"
                type="range"
                min="0"
                max="100"
                value={WiperWidth}
                onChange={changeWiperWidth}
                style={{
                  width: "100px",
                  background: "#ddd", // Background color of the track
                  height: "5px", // Height of the track
                  borderRadius: "5px", // Round edges for the track
                  outline: "none", // Remove outline
                }}
              />
            </div>
          </>
        )}
        {/* 2. Edits for text*/}
        {selectedTool === "Text" && (
          <div className="absolute top-16 space-x-2 bg-white shadow-xl p-3 rounded-lg flex justify-center items-center">
            <input
              className="text-black rounded-color-input"
              type="color"
              value={textProps.color}
              onChange={handleTextColorChange}
            />
            <select
              onChange={(e) => handleFontStyleChange(e)}
              className="p-1 rounded-lg text-xs border border-purple-500"
              defaultValue={"Normal"}
            >
              <option defaultValue={"Normal"}>Normal</option>
              <option value={"Bold"}>Bold</option>
              <option value={"Italic"}>Italic</option>
              <option value={"Underline"}>Underline</option>
              <option value={"StrikeThrough"}>Strike Through</option>
              <option value={"Uppercase"}>Uppercase</option>
              <option value={"Lowercase"}>Lowercase</option>
              <option value={"Capitalize"}>Capitalize</option>
            </select>

            <input
              className="text-black w-11 border rounded-md"
              type="number"
              value={textProps.fontSize}
              onChange={handleTextSizeChange}
              min="1"
              max="100"
            />
            {/* Font Family its options */}
            <FontFamilyAndText
              canvas={canvas}
              setopenSideBarSpacing={setOpenSideBarSpacing}
              setopenSideBarFontFamily={setopenSideBarFontFamily}
              openSideBarFontFamily={openSideBarFontFamily}
              setopenSideBarStroke={setopenSideBarStroke}
            />
            {/* Text Spacing its options */}
            <TextSpacings
              canvas={canvas}
              setopenSideBarSpacing={setOpenSideBarSpacing}
              openSideBarSpacing={openSideBarSpacing}
              setopenSideBarFontFamily={setopenSideBarFontFamily}
              setopenSideBarStroke={setopenSideBarStroke}
            />
            {/* Font Stroke its options */}
            <FontStrokeOptions
              canvas={canvas}
              setopenSideBarSpacing={setOpenSideBarSpacing}
              setopenSideBarFontFamily={setopenSideBarFontFamily}
              openSideBarStroke={openSideBarStroke}
              setopenSideBarStroke={setopenSideBarStroke}
            />
          </div>
        )}
        {/* shape edits */}
        {selectedTool === "Shapes" && (
          <>
            <div className="absolute top-16 space-x-2 bg-white shadow-xl p-3 rounded-lg flex justify-center items-center">
              <input
                className="text-black rounded-color-input"
                type="color"
                value={DrawColor}
                onChange={(e) => handleDrawColor(e)}
              />
              <select
                onChange={(e) => setdrawColorSelectType(e.target.value)}
                className="p-1 rounded-lg text-xs border border-purple-500"
                defaultValue={"stroke"}
              >
                <option value={"stroke"}>Border</option>
                <option value={"fill"}>Fill</option>
              </select>

              <input
                type="range"
                min="0"
                max="15"
                // value={value}
                onChange={changeStrokeSize}
                style={{
                  width: "30px",
                  background: "#ddd", // Background color of the track
                  height: "5px", // Height of the track
                  borderRadius: "5px", // Round edges for the track
                  outline: "none", // Remove outline
                }}
              />
            </div>
          </>
        )}
      </div>
      <div className="flex relative">
        <div>
          <div>
            {/* save drawing */}
            {/* <button
              onClick={handleSaveDrawing}
              className="fixed top-0 z-20 right-10 rounded-l-md p-2 bg-gray-800 text-white hover:bg-yellow-300 hover:text-black"
            >
              <svg
                xmlns="http://www.w3.org/2000/svg"
                fill="none"
                viewBox="0 0 24 24"
                strokeWidth="1.5"
                stroke="currentColor"
                className="size-5"
              >
                <path
                  strokeLinecap="round"
                  strokeLinejoin="round"
                  d="M3.75 9.776c.112-.017.227-.026.344-.026h15.812c.117 0 .232.009.344.026m-16.5 0a2.25 2.25 0 0 0-1.883 2.542l.857 6a2.25 2.25 0 0 0 2.227 1.932H19.05a2.25 2.25 0 0 0 2.227-1.932l.857-6a2.25 2.25 0 0 0-1.883-2.542m-16.5 0V6A2.25 2.25 0 0 1 6 3.75h3.879a1.5 1.5 0 0 1 1.06.44l2.122 2.12a1.5 1.5 0 0 0 1.06.44H18A2.25 2.25 0 0 1 20.25 9v.776"
                />
              </svg>
            </button> */}
            {/* Download drawings */}
            <button
              type="button"
              onClick={toggleDropdown}
              className="fixed top-2 right-2 z-50 inline-flex justify-center text-white p-1 bg-purple-700 hover:bg-purple-800 focus:ring-4 focus:ring-blue-300 font-medium rounded-full text-sm px-4 py-3 me-2 mb-2 opacity-90 dark:hover:bg-purple-700 focus:outline-none dark:focus:ring-blue-800"
            >
              <span>Save File</span>

              {isOpen && (
                <div className="relative inline-block text-left my-3">
                  <div
                    className="origin-top-right absolute right-0 mt-2 w-48 rounded-md shadow-lg bg-white ring-1 ring-black ring-opacity-5"
                    role="menu"
                    aria-orientation="vertical"
                    aria-labelledby="options-menu"
                  >
                    <div className="py-1" role="none">
                      <button
                        onClick={() => handleDownload("jpg")}
                        className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 hover:text-gray-900"
                        role="menuitem"
                      >
                        JPG
                      </button>
                      <button
                        onClick={() => handleDownload("png")}
                        className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 hover:text-gray-900"
                        role="menuitem"
                      >
                        PNG
                      </button>
                      <button
                        onClick={() => ClearCanvas()}
                        className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 hover:text-gray-900"
                        role="menuitem"
                      >
                        Clear Canvas
                      </button>
                      {/* <button
                        onClick={() => handleDownload("pdf")}
                        className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 hover:text-gray-900"
                        role="menuitem"
                      >
                        PDF
                      </button> */}
                      {/* <button
                        onClick={() => handleDownload("svg")}
                        className="block w-full text-left px-4 py-2 text-sm text-gray-700 hover:bg-gray-100 hover:text-gray-900"
                        role="menuitem"
                      >
                        SVG
                      </button> */}
                    </div>
                  </div>
                </div>
              )}
            </button>
          </div>

          {/* <button onClick={handleLoadDrawing} className="absolute top-[55%] left-0  w-full p-2 bg-yellow-500 hover:bg-yellow-400 rounded">
      Load Drawing
    </button>
    <button onClick={handleExportAsImage} className="absolute top-[60%] left-0  w-full p-2 bg-purple-500 hover:bg-purple-400 rounded">
      Export as PNG
    </button> */}

          <div className="fixed mt-14 overflow-x-scroll px-1 h-screen m-auto z-30 flex flex-col bg-gray-50">
            {tools.map((tool, index) => (
              <div key={index} className="relative p-1 py-2 text-center">
                <button
                  key={tool.name}
                  onClick={() => handleSelectTool(tool.name)}
                >
                  <span className="flex">
                    <span
                      className={`p-2 rounded-lg hover:bg-gray-100 flex flex-col justify-center items-center space-y-2 ${
                        selectedTool === tool.name &&
                        "text-gray shadow-lg bg-gray-200"
                      }`}
                    >
                      <span> {tool.icon}</span>
                      <span className="text-sm m-auto">{tool.name}</span>
                    </span>
                  </span>
                </button>

                {
                  <div className="fixed bottom-0 left-20 p-2 flex z-50 opacity-15">
                    <button
                      className=" p-2 hover:bg-orange-600"
                      onClick={() => handleZoom(true)}
                    >
                      {zoomIn}
                    </button>
                    <button
                      className=" p-2  hover:bg-orange-600"
                      onClick={() => handleZoom(false)}
                    >
                      {zoomout}
                    </button>
                  </div>
                }
                {/* 2.shape tools */}
                {selectedTool === "Shapes" && activeBlock.shapes && (
                  <div className="fixed left-[75px] top-[170px] p-2 flex flex-col z-50 bg-white">
                    {[
                      { tool: "Rectangle", icon: "▭" },
                      { tool: "Circle", icon: "○" },
                      { tool: "Triangle", icon: "△" },
                    ].map((tool, index) => (
                      <button
                        key={index}
                        className={`w-full shadow-md hover:text-white hover:bg-purple-500 p-3 ${
                          tool === selectedShapeTool &&
                          "bg-purple-500 text-white"
                        }`}
                        onClick={() => handleDrawShapes(tool.tool)}
                      >
                        {tool.icon}
                      </button>
                    ))}
                  </div>
                )}

                {/* fill */}
                {selectedTool === "Fill" && activeBlock.fill && (
                  <div className="fixed left-[75px] opacity-20 top-52 p-2 flex flex-col z-50">
                    <div className="p-4 mb-2 bg-white shadow-md rounded border border-purple-500">
                      <select
                        value={fillType}
                        onChange={(e) => changefillcolor(e)}
                        className="w-full mb-2 text-black border rounded-md"
                      >
                        <option value="solid">Solid</option>
                        <option value="gradient">Gradient</option>
                      </select>
                      {fillType === "solid" && (
                        <div>
                          <label className="block mb-2">Fill Color</label>
                          <input
                            type="color"
                            value={fillColor}
                            onChange={(e) => setFillColor(e.target.value)}
                            className="w-full mb-2 rounded-color-input"
                          />
                        </div>
                      )}
                      {fillType === "gradient" && (
                        <div>
                          <p>Select Two Colors</p>
                          <input
                            type="color"
                            value={gradientStartColor}
                            onChange={(e) =>
                              setGradientStartColor(e.target.value)
                            }
                            className="w-full mb-2 rounded-color-input"
                          />
                          <input
                            type="color"
                            value={gradientEndColor}
                            onChange={(e) =>
                              setGradientEndColor(e.target.value)
                            }
                            className="w-full mb-2 rounded-color-input"
                          />
                        </div>
                      )}
                      <button
                        onTouchStart={handleBackgroundFill}
                        onClick={handleBackgroundFill}
                        className="w-full p-2 bg-purple-500 hover:bg-purple-600 rounded-md focus:ring text-white text-sm"
                      >
                        Apply Background
                      </button>
                    </div>
                  </div>
                )}
                {/* Image tool */}
                {tool.name === selectedTool && (
                  <div className="fixed left-[75px] opacity-90 top-80 p-2 z-50">
                    {selectedTool === "Image" && isImage && (
                      <>
                        <form className="max-w-sm mx-auto p-4 rounded-md shadow-lg border border-purple-300">
                          <select
                            ref={selectRef}
                            defaultValue={"Select"}
                            onChange={(e) => {
                              applyImageEffects(e.target.value);
                            }}
                            className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg block w-full p-2.5 dark:placeholder-gray-400 dark:text-white mb-3"
                          >
                            <option selected>Choose a Filter</option>
                            <option value="reset">Reset</option>
                            <option value="grayscale">Grayscale</option>
                            <option value="invert">Invert</option>
                            <option value="sepia">Sepia</option>
                            <option value="brightness">Brightness</option>
                            <option value="contrast">Contrast</option>
                            <option value="blur">Blur</option>
                          </select>

                          <div className="flex justify-center items-center space-x-4">
                            <input
                              type="checkbox"
                              checked={lockImageLayer}
                              onChange={(e) => setImageLayerFn(e)}
                              className="w-6 h-6"
                            />
                            <p className="mb-2">Lock Image Layer</p>
                          </div>
                        </form>
                      </>
                    )}
                  </div>
                )}
                {/* layers */}
                {tool.name === selectedTool && (
                  <div className="fixed left-[110px] top-[270px] z-50">
                    <div className="flex-1 overflow-y-auto p-2 broder border-yellow-300 rounded-md">
                      {layers.map((layer) => (
                        <div
                          key={layer.id}
                          className={`p-2 rounded ${
                            selectedLayer === layer
                              ? "bg-yellow-600"
                              : "bg-yellow-700 hover:bg-yellow-600"
                          }`}
                        >
                          <button
                            className="text-left w-full"
                            onClick={() => selectLayer(layer)}
                          >
                            Layer {layers.indexOf(layer) + 1}
                          </button>
                        </div>
                      ))}
                      {/* Layer actions */}
                      {selectedLayer && (
                        <div className="p-4">
                          <button
                            className="p-2 mb-2 bg-red-500 hover:bg-red-400 rounded"
                            onClick={deleteLayer}
                          >
                            Delete Layer
                          </button>
                          <button
                            className="p-2 mb-2 bg-gray-700 hover:bg-gray-600 rounded"
                            onClick={moveLayerUp}
                          >
                            Move Layer Up
                          </button>
                          <button
                            className="p-2 mb-2 bg-gray-700 hover:bg-gray-600 rounded"
                            onClick={moveLayerDown}
                          >
                            Move Layer Down
                          </button>
                          <button
                            className="p-2 mb-2 bg-gray-700 hover:bg-gray-600 rounded"
                            onClick={toggleLayerVisibility}
                          >
                            {selectedLayer.visible
                              ? "Hide Layer"
                              : "Show Layer"}
                          </button>
                          <button
                            className="p-2 mb-2 bg-gray-700 hover:bg-gray-600 rounded"
                            onClick={toggleLayerLock}
                          >
                            {selectedLayer.lockMovementX
                              ? "Unlock Layer"
                              : "Lock Layer"}
                          </button>
                        </div>
                      )}
                    </div>
                  </div>
                )}
                {/* Crop / resize canvas tool */}
                {selectedTool === "Resize" && (
                  <div className="fixed left-[77px] text-sm w-72 h-60 overflow-scroll top-[70px] z-50 bg-white shadow-sm rounded-lg p-2">
                    <div className="space-y-4">
                      <div>
                        <h3 className="font-bold mb-2">Social Media</h3>
                        {renderSizeButtons("socialMedia")}
                      </div>
                      <div>
                        <h3 className="font-bold mb-2">Digital Devices</h3>
                        {renderSizeButtons("digitalDevices")}
                      </div>
                      <div>
                        <h3 className="font-bold mb-2">Paper</h3>
                        {renderSizeButtons("paper")}
                      </div>
                      <div>
                        <h3 className="font-bold mb-2">Custom Size</h3>
                        <form onSubmit={applyCustomSize} className="space-y-2">
                          <input
                            type="number"
                            name="width"
                            value={customSize.width}
                            onChange={handleCustomSizeChange}
                            placeholder="Width"
                            className="w-full p-2 border rounded"
                          />
                          <input
                            type="number"
                            name="height"
                            value={customSize.height}
                            onChange={handleCustomSizeChange}
                            placeholder="Height"
                            className="w-full p-2 border rounded"
                          />
                          <button
                            type="submit"
                            className="w-full p-2 bg-purple-500 text-white rounded hover:bg-purple-600"
                          >
                            Apply Custom Size
                          </button>
                        </form>
                      </div>
                    </div>
                  </div>
                )}
              </div>
            ))}
          </div>
          {selectedObject && (
            <div className="p-4">
              <button
                className="w-full p-2 mb-2 bg-red-500 hover:bg-red-400 rounded"
                onClick={handleDeleteObject}
              >
                Delete
              </button>
              <button
                className="w-full p-2 mb-2 bg-purple-500 hover:bg-purple-400 rounded"
                onClick={handleRotateObject}
              >
                Rotate
              </button>
              {/* Add more object-specific actions here */}
            </div>
          )}
        </div>
        <div className="flex-1">
          <div className="flex justify-center h-1/2 w-full">
            <div className="w-full h-screen overflow-hidden flex justify-center items-center">
              {/* Direct conditions are not allowed in jsx, while there is needed a parent div or element*/}
              <div className="relative">
                {borderCanvas ? (
                  <div className="flex flex-col p-3 space-y-2">
                    <button
                      className="bg-black rounded-full text-white p-2 text-lg font-bold"
                      onClick={removeBackgroundColor}
                    >
                      <svg
                        xmlns="http://www.w3.org/2000/svg"
                        fill="none"
                        viewBox="0 0 24 24"
                        strokeWidth="1.5"
                        stroke="currentColor"
                        className="size-5"
                      >
                        <path
                          strokeLinecap="round"
                          strokeLinejoin="round"
                          d="m14.74 9-.346 9m-4.788 0L9.26 9m9.968-3.21c.342.052.682.107 1.022.166m-1.022-.165L18.16 19.673a2.25 2.25 0 0 1-2.244 2.077H8.084a2.25 2.25 0 0 1-2.244-2.077L4.772 5.79m14.456 0a48.108 48.108 0 0 0-3.478-.397m-12 .562c.34-.059.68-.114 1.022-.165m0 0a48.11 48.11 0 0 1 3.478-.397m7.5 0v-.916c0-1.18-.91-2.164-2.09-2.201a51.964 51.964 0 0 0-3.32 0c-1.18.037-2.09 1.022-2.09 2.201v.916m7.5 0a48.667 48.667 0 0 0-7.5 0"
                        />
                      </svg>
                    </button>
                    <button
                      className="bg-black rounded-full text-white p-2 text-lg font-bold"
                      onClick={() => {
                        setborderCanvas(false);
                      }}
                    >
                      <svg
                        xmlns="http://www.w3.org/2000/svg"
                        fill="none"
                        viewBox="0 0 24 24"
                        strokeWidth="1.5"
                        stroke="currentColor"
                        className="size-5"
                      >
                        <path
                          strokeLinecap="round"
                          strokeLinejoin="round"
                          d="m4.5 12.75 6 6 9-13.5"
                        />
                      </svg>
                    </button>
                  </div>
                ) : (
                  ""
                )}
              </div>
              <canvas
                id="canvas"
                ref={canvasRef}
                style={{
                  border: borderCanvas ? "2px solid yellow" : "none",
                  border: "1px solid lightgray",
                }}
              />
            </div>
          </div>
        </div>
      </div>
    </>
  );
};

export default MainCanvas;
